# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class DomainKeyValueResourceConfiguration(Model):
    """Domain key value resource configuration (domain.KeyValueResourceConfiguration)

    Properties:
        cluster_name: (clusterName) REQUIRED str

        max_data_storage_gb: (maxDataStorageGB) REQUIRED int

        max_ecpu_per_second: (maxECPUPerSecond) REQUIRED int

        profile_name: (profileName) REQUIRED str
    """

    # region fields

    cluster_name: str  # REQUIRED
    max_data_storage_gb: int  # REQUIRED
    max_ecpu_per_second: int  # REQUIRED
    profile_name: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_cluster_name(self, value: str) -> DomainKeyValueResourceConfiguration:
        self.cluster_name = value
        return self

    def with_max_data_storage_gb(
        self, value: int
    ) -> DomainKeyValueResourceConfiguration:
        self.max_data_storage_gb = value
        return self

    def with_max_ecpu_per_second(
        self, value: int
    ) -> DomainKeyValueResourceConfiguration:
        self.max_ecpu_per_second = value
        return self

    def with_profile_name(self, value: str) -> DomainKeyValueResourceConfiguration:
        self.profile_name = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "cluster_name"):
            result["clusterName"] = str(self.cluster_name)
        elif include_empty:
            result["clusterName"] = ""
        if hasattr(self, "max_data_storage_gb"):
            result["maxDataStorageGB"] = int(self.max_data_storage_gb)
        elif include_empty:
            result["maxDataStorageGB"] = 0
        if hasattr(self, "max_ecpu_per_second"):
            result["maxECPUPerSecond"] = int(self.max_ecpu_per_second)
        elif include_empty:
            result["maxECPUPerSecond"] = 0
        if hasattr(self, "profile_name"):
            result["profileName"] = str(self.profile_name)
        elif include_empty:
            result["profileName"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        cluster_name: str,
        max_data_storage_gb: int,
        max_ecpu_per_second: int,
        profile_name: str,
        **kwargs,
    ) -> DomainKeyValueResourceConfiguration:
        instance = cls()
        instance.cluster_name = cluster_name
        instance.max_data_storage_gb = max_data_storage_gb
        instance.max_ecpu_per_second = max_ecpu_per_second
        instance.profile_name = profile_name
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DomainKeyValueResourceConfiguration:
        instance = cls()
        if not dict_:
            return instance
        if "clusterName" in dict_ and dict_["clusterName"] is not None:
            instance.cluster_name = str(dict_["clusterName"])
        elif include_empty:
            instance.cluster_name = ""
        if "maxDataStorageGB" in dict_ and dict_["maxDataStorageGB"] is not None:
            instance.max_data_storage_gb = int(dict_["maxDataStorageGB"])
        elif include_empty:
            instance.max_data_storage_gb = 0
        if "maxECPUPerSecond" in dict_ and dict_["maxECPUPerSecond"] is not None:
            instance.max_ecpu_per_second = int(dict_["maxECPUPerSecond"])
        elif include_empty:
            instance.max_ecpu_per_second = 0
        if "profileName" in dict_ and dict_["profileName"] is not None:
            instance.profile_name = str(dict_["profileName"])
        elif include_empty:
            instance.profile_name = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DomainKeyValueResourceConfiguration]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DomainKeyValueResourceConfiguration]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        DomainKeyValueResourceConfiguration,
        List[DomainKeyValueResourceConfiguration],
        Dict[Any, DomainKeyValueResourceConfiguration],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "clusterName": "cluster_name",
            "maxDataStorageGB": "max_data_storage_gb",
            "maxECPUPerSecond": "max_ecpu_per_second",
            "profileName": "profile_name",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "clusterName": True,
            "maxDataStorageGB": True,
            "maxECPUPerSecond": True,
            "profileName": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class NosqlresourceNoSQLResourceConfiguration(Model):
    """Nosqlresource no SQL resource configuration (nosqlresource.NoSQLResourceConfiguration)

    Properties:
        max_dcu: (maxDCU) REQUIRED float

        min_dcu: (minDCU) REQUIRED float

        profile_name: (profileName) OPTIONAL str
    """

    # region fields

    max_dcu: float  # REQUIRED
    min_dcu: float  # REQUIRED
    profile_name: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_max_dcu(self, value: float) -> NosqlresourceNoSQLResourceConfiguration:
        self.max_dcu = value
        return self

    def with_min_dcu(self, value: float) -> NosqlresourceNoSQLResourceConfiguration:
        self.min_dcu = value
        return self

    def with_profile_name(self, value: str) -> NosqlresourceNoSQLResourceConfiguration:
        self.profile_name = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "max_dcu"):
            result["maxDCU"] = float(self.max_dcu)
        elif include_empty:
            result["maxDCU"] = 0.0
        if hasattr(self, "min_dcu"):
            result["minDCU"] = float(self.min_dcu)
        elif include_empty:
            result["minDCU"] = 0.0
        if hasattr(self, "profile_name"):
            result["profileName"] = str(self.profile_name)
        elif include_empty:
            result["profileName"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        max_dcu: float,
        min_dcu: float,
        profile_name: Optional[str] = None,
        **kwargs,
    ) -> NosqlresourceNoSQLResourceConfiguration:
        instance = cls()
        instance.max_dcu = max_dcu
        instance.min_dcu = min_dcu
        if profile_name is not None:
            instance.profile_name = profile_name
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> NosqlresourceNoSQLResourceConfiguration:
        instance = cls()
        if not dict_:
            return instance
        if "maxDCU" in dict_ and dict_["maxDCU"] is not None:
            instance.max_dcu = float(dict_["maxDCU"])
        elif include_empty:
            instance.max_dcu = 0.0
        if "minDCU" in dict_ and dict_["minDCU"] is not None:
            instance.min_dcu = float(dict_["minDCU"])
        elif include_empty:
            instance.min_dcu = 0.0
        if "profileName" in dict_ and dict_["profileName"] is not None:
            instance.profile_name = str(dict_["profileName"])
        elif include_empty:
            instance.profile_name = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, NosqlresourceNoSQLResourceConfiguration]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[NosqlresourceNoSQLResourceConfiguration]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        NosqlresourceNoSQLResourceConfiguration,
        List[NosqlresourceNoSQLResourceConfiguration],
        Dict[Any, NosqlresourceNoSQLResourceConfiguration],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "maxDCU": "max_dcu",
            "minDCU": "min_dcu",
            "profileName": "profile_name",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "maxDCU": True,
            "minDCU": True,
            "profileName": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import AccountcommonClientPermissions
from ..models import AccountcommonClientPermissionsV3
from ..models import ClientmodelClientCreateRequest
from ..models import ClientmodelClientCreationResponse
from ..models import ClientmodelClientCreationV3Request
from ..models import ClientmodelClientResponse
from ..models import ClientmodelClientUpdateRequest
from ..models import ClientmodelClientUpdateSecretRequest
from ..models import ClientmodelClientUpdateV3Request
from ..models import ClientmodelClientV3Response
from ..models import ClientmodelClientsUpdateRequestV3
from ..models import ClientmodelClientsV3Response
from ..models import ClientmodelV3ClientUpdateSecretRequest
from ..models import RestErrorResponse

from ..operations.clients import AddClientPermission
from ..operations.clients import AdminAddClientPermissionsV3
from ..operations.clients import AdminBulkUpdateClientsV3
from ..operations.clients import AdminCreateClientV3
from ..operations.clients import AdminDeleteClientPermissionV3
from ..operations.clients import AdminDeleteClientV3
from ..operations.clients import AdminGetClientsByNamespaceV3
from ..operations.clients import AdminGetClientsbyNamespacebyIDV3
from ..operations.clients import AdminUpdateClientPermissionV3
from ..operations.clients import AdminUpdateClientSecretV3
from ..operations.clients import AdminUpdateClientV3
from ..operations.clients import CreateClient
from ..operations.clients import CreateClientByNamespace
from ..operations.clients import DeleteClient
from ..operations.clients import DeleteClientByNamespace
from ..operations.clients import DeleteClientPermission
from ..operations.clients import GetClient
from ..operations.clients import GetClients
from ..operations.clients import GetClientsbyNamespace
from ..operations.clients import UpdateClient
from ..operations.clients import UpdateClientPermission
from ..operations.clients import UpdateClientSecret


@deprecated
@same_doc_as(AddClientPermission)
def add_client_permission(
    action: int,
    client_id: str,
    resource: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add Client Permission (AddClientPermission)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions [POST]_**

    Properties:
        url: /iam/clients/{clientId}/clientpermissions/{resource}/{action}

        method: POST

        tags: ["Clients"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        action: (action) REQUIRED int in path

        client_id: (clientId) REQUIRED str in path

        resource: (resource) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = AddClientPermission.create(
        action=action,
        client_id=client_id,
        resource=resource,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(AddClientPermission)
async def add_client_permission_async(
    action: int,
    client_id: str,
    resource: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add Client Permission (AddClientPermission)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions [POST]_**

    Properties:
        url: /iam/clients/{clientId}/clientpermissions/{resource}/{action}

        method: POST

        tags: ["Clients"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        action: (action) REQUIRED int in path

        client_id: (clientId) REQUIRED str in path

        resource: (resource) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = AddClientPermission.create(
        action=action,
        client_id=client_id,
        resource=resource,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminAddClientPermissionsV3)
def admin_add_client_permissions_v3(
    body: AccountcommonClientPermissionsV3,
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add Client Permissions (AdminAddClientPermissionsV3)

    **Note for Multi Tenant Mode:**
    Studio admin & game admin can only update custom permissions:
    * The allowed prefix are: 1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}*** 2. ***CUSTOM:NAMESPACE:{namespace}***

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED AccountcommonClientPermissionsV3 in body

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminAddClientPermissionsV3.create(
        body=body,
        client_id=client_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminAddClientPermissionsV3)
async def admin_add_client_permissions_v3_async(
    body: AccountcommonClientPermissionsV3,
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add Client Permissions (AdminAddClientPermissionsV3)

    **Note for Multi Tenant Mode:**
    Studio admin & game admin can only update custom permissions:
    * The allowed prefix are: 1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}*** 2. ***CUSTOM:NAMESPACE:{namespace}***

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED AccountcommonClientPermissionsV3 in body

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminAddClientPermissionsV3.create(
        body=body,
        client_id=client_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminBulkUpdateClientsV3)
def admin_bulk_update_clients_v3(
    body: ClientmodelClientsUpdateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk Update Clients (AdminBulkUpdateClientsV3)

    Updates multiple OAuth 2.0 clients.
    Specify only the fields you want to update in the request payload, e.g. {"clientName":"E-commerce", "baseUri":"https://example.net"}

    **Note for Multi Tenant Mode (Confidential Client):**
    Only Super Admins can set permission with resource and action.
    Studio admin & game admin can only set permission through the permission module.
    If Studio admin & game admin need to set custom permission string, they must use the non-bulk API.

    **Fields Description:**
    - **clientName** : The client name. It should not be empty if the field exists in the body. e.g E-commerce
    - **namespace** : The namespace where the client lives. e.g sample-game
    - **redirectUri** : Contains the redirect URI used in OAuth callback. It should not be empty if the field exists in the body. e.g https://example.net/platform
    - **audiences** : List of target client IDs who is intended to receive the token. e.g ["eaaa65618fe24293b00a61454182b435", "40073ee9bc3446d3a051a71b48509a5d"]
    - **baseUri** : A base URI of the application. It is used in the audience checking for making sure the token is used by the right resource server. Required if the application type is a server. e.g https://example.net/platform
    - **clientPermissions** : Contains the client's permissions
    - **deletable** : The flag to identify whether client is deletable (optional). e.g. true
    - **clientPlatform** : available client platform (optional). default value: "".
    - Playstation
    - Xbox
    - Steam
    - Epic
    - IOS
    - GooglePlay
    - Nintendo
    - Oculus
    - **twoFactorEnabled**: The flag to indicate whether 2FA validation is enable for this client. default value: false
    - **oauthAccessTokenExpiration**: a configurable expiration time for **access_token**, default value: 0 (mean fetch value from environment variable)
    - **oauthRefreshTokenExpiration**: a configurable expiration time for **refresh_token**, default value: 0 (mean fetch value from environment variable)
    - **oauthAccessTokenExpirationTimeUnit**: a configurable expiration time unit for **access_token**, will use previous value if not specified
    - **oauthRefreshTokenExpirationTimeUnit**: a configurable expiration time unit for **refresh_token**, will use previous value if not specified
    - **skipLoginQueue**: a flag to indicate whether this client should be exempted from login queue or not

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientsUpdateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminBulkUpdateClientsV3.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminBulkUpdateClientsV3)
async def admin_bulk_update_clients_v3_async(
    body: ClientmodelClientsUpdateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk Update Clients (AdminBulkUpdateClientsV3)

    Updates multiple OAuth 2.0 clients.
    Specify only the fields you want to update in the request payload, e.g. {"clientName":"E-commerce", "baseUri":"https://example.net"}

    **Note for Multi Tenant Mode (Confidential Client):**
    Only Super Admins can set permission with resource and action.
    Studio admin & game admin can only set permission through the permission module.
    If Studio admin & game admin need to set custom permission string, they must use the non-bulk API.

    **Fields Description:**
    - **clientName** : The client name. It should not be empty if the field exists in the body. e.g E-commerce
    - **namespace** : The namespace where the client lives. e.g sample-game
    - **redirectUri** : Contains the redirect URI used in OAuth callback. It should not be empty if the field exists in the body. e.g https://example.net/platform
    - **audiences** : List of target client IDs who is intended to receive the token. e.g ["eaaa65618fe24293b00a61454182b435", "40073ee9bc3446d3a051a71b48509a5d"]
    - **baseUri** : A base URI of the application. It is used in the audience checking for making sure the token is used by the right resource server. Required if the application type is a server. e.g https://example.net/platform
    - **clientPermissions** : Contains the client's permissions
    - **deletable** : The flag to identify whether client is deletable (optional). e.g. true
    - **clientPlatform** : available client platform (optional). default value: "".
    - Playstation
    - Xbox
    - Steam
    - Epic
    - IOS
    - GooglePlay
    - Nintendo
    - Oculus
    - **twoFactorEnabled**: The flag to indicate whether 2FA validation is enable for this client. default value: false
    - **oauthAccessTokenExpiration**: a configurable expiration time for **access_token**, default value: 0 (mean fetch value from environment variable)
    - **oauthRefreshTokenExpiration**: a configurable expiration time for **refresh_token**, default value: 0 (mean fetch value from environment variable)
    - **oauthAccessTokenExpirationTimeUnit**: a configurable expiration time unit for **access_token**, will use previous value if not specified
    - **oauthRefreshTokenExpirationTimeUnit**: a configurable expiration time unit for **refresh_token**, will use previous value if not specified
    - **skipLoginQueue**: a flag to indicate whether this client should be exempted from login queue or not

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientsUpdateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminBulkUpdateClientsV3.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminCreateClientV3)
def admin_create_client_v3(
    body: ClientmodelClientCreationV3Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Client (AdminCreateClientV3)

    Creates a new OAuth 2.0 client.
    A new client automatically granted with these scopes: commerce, account, analytics, publishing, social.
    **Note for Multi Tenant Mode (Confidential Client):**
    Super admin can set any valid format permission with resource & action.
    Studio admin & game admin can only set permission with these 2 ways:
    * Permission module, available from _GET /iam/v3/admin/clientConfig/permissions_
    * Fixed prefix custom permissions, the allowed prefix are:
    1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}***
    2. ***CUSTOM:NAMESPACE:{namespace}***

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientCreationV3Request in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ClientmodelClientV3Response (Created)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        409: Conflict - RestErrorResponse (10364: client exists)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateClientV3.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateClientV3)
async def admin_create_client_v3_async(
    body: ClientmodelClientCreationV3Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Client (AdminCreateClientV3)

    Creates a new OAuth 2.0 client.
    A new client automatically granted with these scopes: commerce, account, analytics, publishing, social.
    **Note for Multi Tenant Mode (Confidential Client):**
    Super admin can set any valid format permission with resource & action.
    Studio admin & game admin can only set permission with these 2 ways:
    * Permission module, available from _GET /iam/v3/admin/clientConfig/permissions_
    * Fixed prefix custom permissions, the allowed prefix are:
    1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}***
    2. ***CUSTOM:NAMESPACE:{namespace}***

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientCreationV3Request in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ClientmodelClientV3Response (Created)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        409: Conflict - RestErrorResponse (10364: client exists)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateClientV3.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminDeleteClientPermissionV3)
def admin_delete_client_permission_v3(
    action: int,
    client_id: str,
    resource: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client Permission (AdminDeleteClientPermissionV3)

    **Note for Multi Tenant Mode:**
    Studio admin & game admin are only allowed to delete custom permissions:
    * The allowed prefix are: 1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}*** 2. ***CUSTOM:NAMESPACE:{namespace}***

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions/{resource}/{action}

        method: DELETE

        tags: ["Clients"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        action: (action) REQUIRED int in path

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        resource: (resource) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteClientPermissionV3.create(
        action=action,
        client_id=client_id,
        resource=resource,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteClientPermissionV3)
async def admin_delete_client_permission_v3_async(
    action: int,
    client_id: str,
    resource: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client Permission (AdminDeleteClientPermissionV3)

    **Note for Multi Tenant Mode:**
    Studio admin & game admin are only allowed to delete custom permissions:
    * The allowed prefix are: 1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}*** 2. ***CUSTOM:NAMESPACE:{namespace}***

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions/{resource}/{action}

        method: DELETE

        tags: ["Clients"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        action: (action) REQUIRED int in path

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        resource: (resource) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteClientPermissionV3.create(
        action=action,
        client_id=client_id,
        resource=resource,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminDeleteClientV3)
def admin_delete_client_v3(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client (AdminDeleteClientV3)

    Deletes the specified OAuth client.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}

        method: DELETE

        tags: ["Clients"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)

        409: Conflict - RestErrorResponse (20009: request_conflict)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteClientV3.create(
        client_id=client_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteClientV3)
async def admin_delete_client_v3_async(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client (AdminDeleteClientV3)

    Deletes the specified OAuth client.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}

        method: DELETE

        tags: ["Clients"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)

        409: Conflict - RestErrorResponse (20009: request_conflict)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteClientV3.create(
        client_id=client_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetClientsByNamespaceV3)
def admin_get_clients_by_namespace_v3(
    client_id: Optional[str] = None,
    client_name: Optional[str] = None,
    client_type: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    skip_login_queue: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Clients by Namespace (AdminGetClientsByNamespaceV3)

    Retrieves all OAuth clients in the namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients

        method: GET

        tags: ["Clients"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        client_id: (clientId) OPTIONAL str in query

        client_name: (clientName) OPTIONAL str in query

        client_type: (clientType) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        skip_login_queue: (skipLoginQueue) OPTIONAL bool in query

    Responses:
        200: OK - ClientmodelClientsV3Response (OK)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetClientsByNamespaceV3.create(
        client_id=client_id,
        client_name=client_name,
        client_type=client_type,
        limit=limit,
        offset=offset,
        skip_login_queue=skip_login_queue,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetClientsByNamespaceV3)
async def admin_get_clients_by_namespace_v3_async(
    client_id: Optional[str] = None,
    client_name: Optional[str] = None,
    client_type: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    skip_login_queue: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Clients by Namespace (AdminGetClientsByNamespaceV3)

    Retrieves all OAuth clients in the namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients

        method: GET

        tags: ["Clients"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        client_id: (clientId) OPTIONAL str in query

        client_name: (clientName) OPTIONAL str in query

        client_type: (clientType) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        skip_login_queue: (skipLoginQueue) OPTIONAL bool in query

    Responses:
        200: OK - ClientmodelClientsV3Response (OK)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetClientsByNamespaceV3.create(
        client_id=client_id,
        client_name=client_name,
        client_type=client_type,
        limit=limit,
        offset=offset,
        skip_login_queue=skip_login_queue,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetClientsbyNamespacebyIDV3)
def admin_get_clientsby_namespaceby_idv3(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Client by Namespace and Client ID (AdminGetClientsbyNamespacebyIDV3)

    Retrieves an OAuth client by namespace and client ID. In multi-tenant mode, the admin portal client is accessible regardless of the namespace path.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}

        method: GET

        tags: ["Clients"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ClientmodelClientV3Response (OK)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetClientsbyNamespacebyIDV3.create(
        client_id=client_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetClientsbyNamespacebyIDV3)
async def admin_get_clientsby_namespaceby_idv3_async(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Client by Namespace and Client ID (AdminGetClientsbyNamespacebyIDV3)

    Retrieves an OAuth client by namespace and client ID. In multi-tenant mode, the admin portal client is accessible regardless of the namespace path.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}

        method: GET

        tags: ["Clients"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ClientmodelClientV3Response (OK)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetClientsbyNamespacebyIDV3.create(
        client_id=client_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateClientPermissionV3)
def admin_update_client_permission_v3(
    body: AccountcommonClientPermissionsV3,
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client Permissions (AdminUpdateClientPermissionV3)

    **Note for Multi Tenant Mode:**
    Studio admin & game admin can only add custom permissions:
    * The allowed prefix are: 1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}*** 2. ***CUSTOM:NAMESPACE:{namespace}***
    * If the request body has new added/delete non-custom permission, the request will fail.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED AccountcommonClientPermissionsV3 in body

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateClientPermissionV3.create(
        body=body,
        client_id=client_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateClientPermissionV3)
async def admin_update_client_permission_v3_async(
    body: AccountcommonClientPermissionsV3,
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client Permissions (AdminUpdateClientPermissionV3)

    **Note for Multi Tenant Mode:**
    Studio admin & game admin can only add custom permissions:
    * The allowed prefix are: 1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}*** 2. ***CUSTOM:NAMESPACE:{namespace}***
    * If the request body has new added/delete non-custom permission, the request will fail.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED AccountcommonClientPermissionsV3 in body

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateClientPermissionV3.create(
        body=body,
        client_id=client_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateClientSecretV3)
def admin_update_client_secret_v3(
    body: ClientmodelV3ClientUpdateSecretRequest,
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client Secret (AdminUpdateClientSecretV3)

    Updates the client secret for the specified OAuth client.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/{clientId}/secret

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelV3ClientUpdateSecretRequest in body

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateClientSecretV3.create(
        body=body,
        client_id=client_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateClientSecretV3)
async def admin_update_client_secret_v3_async(
    body: ClientmodelV3ClientUpdateSecretRequest,
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client Secret (AdminUpdateClientSecretV3)

    Updates the client secret for the specified OAuth client.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/{clientId}/secret

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelV3ClientUpdateSecretRequest in body

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateClientSecretV3.create(
        body=body,
        client_id=client_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateClientV3)
def admin_update_client_v3(
    body: ClientmodelClientUpdateV3Request,
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client (AdminUpdateClientV3)

    Updates an OAuth 2.0 client. Specify only the fields you want to update in the request payload.

    **Note for Multi Tenant Mode (Confidential Client):**
    Super admin can set any valid format permission with resource & action.
    Studio admin & game admin can only set permission with these 2 ways:
    * Permission module, available from _GET /iam/v3/admin/clientConfig/permissions_
    * Fixed prefix custom permissions, the allowed prefix are:
    1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}***
    2. ***CUSTOM:NAMESPACE:{namespace}***

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}

        method: PATCH

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientUpdateV3Request in body

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ClientmodelClientV3Response (OK)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateClientV3.create(
        body=body,
        client_id=client_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateClientV3)
async def admin_update_client_v3_async(
    body: ClientmodelClientUpdateV3Request,
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client (AdminUpdateClientV3)

    Updates an OAuth 2.0 client. Specify only the fields you want to update in the request payload.

    **Note for Multi Tenant Mode (Confidential Client):**
    Super admin can set any valid format permission with resource & action.
    Studio admin & game admin can only set permission with these 2 ways:
    * Permission module, available from _GET /iam/v3/admin/clientConfig/permissions_
    * Fixed prefix custom permissions, the allowed prefix are:
    1. ***CUSTOM:ADMIN:NAMESPACE:{namespace}***
    2. ***CUSTOM:NAMESPACE:{namespace}***

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/clients/{clientId}

        method: PATCH

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientUpdateV3Request in body

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ClientmodelClientV3Response (OK)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10365: client not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateClientV3.create(
        body=body,
        client_id=client_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(CreateClient)
def create_client(
    body: ClientmodelClientCreateRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Client (CreateClient)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients [POST]_**

    **Note for Multi Tenant Mode (Confidential Client):**
    Only Super admin can set permission with resource & action.
    Studio admin & game admin need set permission with permission module in _/v3/admin/namespaces/{namespace}/clients [POST]_.

    Properties:
        url: /iam/clients

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientCreateRequest in body

    Responses:
        201: Created - ClientmodelClientCreationResponse (Created)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        409: Conflict - (Client exists)
    """
    request = CreateClient.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(CreateClient)
async def create_client_async(
    body: ClientmodelClientCreateRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Client (CreateClient)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients [POST]_**

    **Note for Multi Tenant Mode (Confidential Client):**
    Only Super admin can set permission with resource & action.
    Studio admin & game admin need set permission with permission module in _/v3/admin/namespaces/{namespace}/clients [POST]_.

    Properties:
        url: /iam/clients

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientCreateRequest in body

    Responses:
        201: Created - ClientmodelClientCreationResponse (Created)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        409: Conflict - (Client exists)
    """
    request = CreateClient.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(CreateClientByNamespace)
def create_client_by_namespace(
    body: ClientmodelClientCreateRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Client (CreateClientByNamespace)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/v3/admin/namespaces/{namespace}/clients [POST]_**

    **Note for Multi Tenant Mode (Confidential Client):**
    Only Super admin can set permission with resource & action.
    Studio admin & game admin need set permission with permission module in _/v3/admin/namespaces/{namespace}/clients [POST]_.

    Properties:
        url: /iam/namespaces/{namespace}/clients

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientCreateRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ClientmodelClientCreationResponse (Created)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        409: Conflict - (Client exists)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateClientByNamespace.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(CreateClientByNamespace)
async def create_client_by_namespace_async(
    body: ClientmodelClientCreateRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Client (CreateClientByNamespace)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/v3/admin/namespaces/{namespace}/clients [POST]_**

    **Note for Multi Tenant Mode (Confidential Client):**
    Only Super admin can set permission with resource & action.
    Studio admin & game admin need set permission with permission module in _/v3/admin/namespaces/{namespace}/clients [POST]_.

    Properties:
        url: /iam/namespaces/{namespace}/clients

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientCreateRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ClientmodelClientCreationResponse (Created)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        409: Conflict - (Client exists)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateClientByNamespace.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(DeleteClient)
def delete_client(
    client_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Delete Client (DeleteClient)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId} [DELETE]_**

    Properties:
        url: /iam/clients/{clientId}

        method: DELETE

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = DeleteClient.create(
        client_id=client_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(DeleteClient)
async def delete_client_async(
    client_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Delete Client (DeleteClient)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId} [DELETE]_**

    Properties:
        url: /iam/clients/{clientId}

        method: DELETE

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = DeleteClient.create(
        client_id=client_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(DeleteClientByNamespace)
def delete_client_by_namespace(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client (DeleteClientByNamespace)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId} [DELETE]_**

    Properties:
        url: /iam/namespaces/{namespace}/clients/{clientId}

        method: DELETE

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteClientByNamespace.create(
        client_id=client_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(DeleteClientByNamespace)
async def delete_client_by_namespace_async(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client (DeleteClientByNamespace)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId} [DELETE]_**

    Properties:
        url: /iam/namespaces/{namespace}/clients/{clientId}

        method: DELETE

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteClientByNamespace.create(
        client_id=client_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(DeleteClientPermission)
def delete_client_permission(
    action: int,
    client_id: str,
    resource: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client Permission (DeleteClientPermission)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions/{resource}/{action} [DELETE]_**

    Properties:
        url: /iam/clients/{clientId}/clientpermissions/{resource}/{action}

        method: DELETE

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        action: (action) REQUIRED int in path

        client_id: (clientId) REQUIRED str in path

        resource: (resource) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = DeleteClientPermission.create(
        action=action,
        client_id=client_id,
        resource=resource,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(DeleteClientPermission)
async def delete_client_permission_async(
    action: int,
    client_id: str,
    resource: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client Permission (DeleteClientPermission)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions/{resource}/{action} [DELETE]_**

    Properties:
        url: /iam/clients/{clientId}/clientpermissions/{resource}/{action}

        method: DELETE

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        action: (action) REQUIRED int in path

        client_id: (clientId) REQUIRED str in path

        resource: (resource) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = DeleteClientPermission.create(
        action=action,
        client_id=client_id,
        resource=resource,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetClient)
def get_client(
    client_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get Client (GetClient)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId} [GET]_**

    Properties:
        url: /iam/clients/{clientId}

        method: GET

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

    Responses:
        200: OK - ClientmodelClientResponse (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = GetClient.create(
        client_id=client_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetClient)
async def get_client_async(
    client_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get Client (GetClient)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId} [GET]_**

    Properties:
        url: /iam/clients/{clientId}

        method: GET

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        client_id: (clientId) REQUIRED str in path

    Responses:
        200: OK - ClientmodelClientResponse (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = GetClient.create(
        client_id=client_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetClients)
def get_clients(x_additional_headers: Optional[Dict[str, str]] = None, **kwargs):
    """Get All Clients (GetClients)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients [GET]_**

    Properties:
        url: /iam/clients

        method: GET

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - List[ClientmodelClientResponse] (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = GetClients.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetClients)
async def get_clients_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get All Clients (GetClients)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients [GET]_**

    Properties:
        url: /iam/clients

        method: GET

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - List[ClientmodelClientResponse] (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = GetClients.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetClientsbyNamespace)
def get_clientsby_namespace(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get clients by namespace (GetClientsbyNamespace)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients [GET]_**

    Properties:
        url: /iam/namespaces/{namespace}/clients

        method: GET

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ClientmodelClientResponse] (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetClientsbyNamespace.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetClientsbyNamespace)
async def get_clientsby_namespace_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get clients by namespace (GetClientsbyNamespace)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients [GET]_**

    Properties:
        url: /iam/namespaces/{namespace}/clients

        method: GET

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ClientmodelClientResponse] (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetClientsbyNamespace.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(UpdateClient)
def update_client(
    body: ClientmodelClientUpdateRequest,
    client_id: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client (UpdateClient)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId} [PATCH]_**

    Properties:
        url: /iam/clients/{clientId}

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientUpdateRequest in body

        client_id: (clientId) REQUIRED str in path

    Responses:
        200: OK - ClientmodelClientResponse (OK)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = UpdateClient.create(
        body=body,
        client_id=client_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(UpdateClient)
async def update_client_async(
    body: ClientmodelClientUpdateRequest,
    client_id: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client (UpdateClient)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/clients/{clientId} [PATCH]_**

    Properties:
        url: /iam/clients/{clientId}

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientUpdateRequest in body

        client_id: (clientId) REQUIRED str in path

    Responses:
        200: OK - ClientmodelClientResponse (OK)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = UpdateClient.create(
        body=body,
        client_id=client_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(UpdateClientPermission)
def update_client_permission(
    body: AccountcommonClientPermissions,
    client_id: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client Permissions (UpdateClientPermission)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions [PUT]_**

    Properties:
        url: /iam/clients/{clientId}/clientpermissions

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED AccountcommonClientPermissions in body

        client_id: (clientId) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = UpdateClientPermission.create(
        body=body,
        client_id=client_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(UpdateClientPermission)
async def update_client_permission_async(
    body: AccountcommonClientPermissions,
    client_id: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client Permissions (UpdateClientPermission)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _iam/v3/admin/namespaces/{namespace}/clients/{clientId}/permissions [PUT]_**

    Properties:
        url: /iam/clients/{clientId}/clientpermissions

        method: POST

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED AccountcommonClientPermissions in body

        client_id: (clientId) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = UpdateClientPermission.create(
        body=body,
        client_id=client_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(UpdateClientSecret)
def update_client_secret(
    body: ClientmodelClientUpdateSecretRequest,
    client_id: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client Secret (UpdateClientSecret)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/{clientId}/secret [PUT]_**

    Properties:
        url: /iam/clients/{clientId}/secret

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientUpdateSecretRequest in body

        client_id: (clientId) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = UpdateClientSecret.create(
        body=body,
        client_id=client_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(UpdateClientSecret)
async def update_client_secret_async(
    body: ClientmodelClientUpdateSecretRequest,
    client_id: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Client Secret (UpdateClientSecret)

    **This endpoint is deprecated.**
    **Endpoint migration guide**
    - **Substitute endpoint: _/iam/v3/admin/namespaces/{namespace}/{clientId}/secret [PUT]_**

    Properties:
        url: /iam/clients/{clientId}/secret

        method: PUT

        tags: ["Clients"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelClientUpdateSecretRequest in body

        client_id: (clientId) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (Invalid request)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - (Data not found)
    """
    request = UpdateClientSecret.create(
        body=body,
        client_id=client_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelGetListOfConfigurationsV2DataItem(Model):
    """Apimodel get list of configurations V2 data item (apimodel.GetListOfConfigurationsV2DataItem)

    Properties:
        config_id: (configId) REQUIRED str

        config_name: (configName) REQUIRED str

        config_type: (configType) REQUIRED str

        deployment_status: (deploymentStatus) REQUIRED str

        editable: (editable) REQUIRED bool

        is_hidden: (isHidden) REQUIRED bool

        value: (value) REQUIRED str

        apply_mask: (applyMask) OPTIONAL bool

        description: (description) OPTIONAL str

        source: (source) OPTIONAL str
    """

    # region fields

    config_id: str  # REQUIRED
    config_name: str  # REQUIRED
    config_type: str  # REQUIRED
    deployment_status: str  # REQUIRED
    editable: bool  # REQUIRED
    is_hidden: bool  # REQUIRED
    value: str  # REQUIRED
    apply_mask: bool  # OPTIONAL
    description: str  # OPTIONAL
    source: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_config_id(self, value: str) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.config_id = value
        return self

    def with_config_name(self, value: str) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.config_name = value
        return self

    def with_config_type(self, value: str) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.config_type = value
        return self

    def with_deployment_status(
        self, value: str
    ) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.deployment_status = value
        return self

    def with_editable(self, value: bool) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.editable = value
        return self

    def with_is_hidden(self, value: bool) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.is_hidden = value
        return self

    def with_value(self, value: str) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.value = value
        return self

    def with_apply_mask(self, value: bool) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.apply_mask = value
        return self

    def with_description(self, value: str) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.description = value
        return self

    def with_source(self, value: str) -> ApimodelGetListOfConfigurationsV2DataItem:
        self.source = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "config_id"):
            result["configId"] = str(self.config_id)
        elif include_empty:
            result["configId"] = ""
        if hasattr(self, "config_name"):
            result["configName"] = str(self.config_name)
        elif include_empty:
            result["configName"] = ""
        if hasattr(self, "config_type"):
            result["configType"] = str(self.config_type)
        elif include_empty:
            result["configType"] = ""
        if hasattr(self, "deployment_status"):
            result["deploymentStatus"] = str(self.deployment_status)
        elif include_empty:
            result["deploymentStatus"] = ""
        if hasattr(self, "editable"):
            result["editable"] = bool(self.editable)
        elif include_empty:
            result["editable"] = False
        if hasattr(self, "is_hidden"):
            result["isHidden"] = bool(self.is_hidden)
        elif include_empty:
            result["isHidden"] = False
        if hasattr(self, "value"):
            result["value"] = str(self.value)
        elif include_empty:
            result["value"] = ""
        if hasattr(self, "apply_mask"):
            result["applyMask"] = bool(self.apply_mask)
        elif include_empty:
            result["applyMask"] = False
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "source"):
            result["source"] = str(self.source)
        elif include_empty:
            result["source"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        config_id: str,
        config_name: str,
        config_type: str,
        deployment_status: str,
        editable: bool,
        is_hidden: bool,
        value: str,
        apply_mask: Optional[bool] = None,
        description: Optional[str] = None,
        source: Optional[str] = None,
        **kwargs,
    ) -> ApimodelGetListOfConfigurationsV2DataItem:
        instance = cls()
        instance.config_id = config_id
        instance.config_name = config_name
        instance.config_type = config_type
        instance.deployment_status = deployment_status
        instance.editable = editable
        instance.is_hidden = is_hidden
        instance.value = value
        if apply_mask is not None:
            instance.apply_mask = apply_mask
        if description is not None:
            instance.description = description
        if source is not None:
            instance.source = source
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelGetListOfConfigurationsV2DataItem:
        instance = cls()
        if not dict_:
            return instance
        if "configId" in dict_ and dict_["configId"] is not None:
            instance.config_id = str(dict_["configId"])
        elif include_empty:
            instance.config_id = ""
        if "configName" in dict_ and dict_["configName"] is not None:
            instance.config_name = str(dict_["configName"])
        elif include_empty:
            instance.config_name = ""
        if "configType" in dict_ and dict_["configType"] is not None:
            instance.config_type = str(dict_["configType"])
        elif include_empty:
            instance.config_type = ""
        if "deploymentStatus" in dict_ and dict_["deploymentStatus"] is not None:
            instance.deployment_status = str(dict_["deploymentStatus"])
        elif include_empty:
            instance.deployment_status = ""
        if "editable" in dict_ and dict_["editable"] is not None:
            instance.editable = bool(dict_["editable"])
        elif include_empty:
            instance.editable = False
        if "isHidden" in dict_ and dict_["isHidden"] is not None:
            instance.is_hidden = bool(dict_["isHidden"])
        elif include_empty:
            instance.is_hidden = False
        if "value" in dict_ and dict_["value"] is not None:
            instance.value = str(dict_["value"])
        elif include_empty:
            instance.value = ""
        if "applyMask" in dict_ and dict_["applyMask"] is not None:
            instance.apply_mask = bool(dict_["applyMask"])
        elif include_empty:
            instance.apply_mask = False
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "source" in dict_ and dict_["source"] is not None:
            instance.source = str(dict_["source"])
        elif include_empty:
            instance.source = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelGetListOfConfigurationsV2DataItem]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelGetListOfConfigurationsV2DataItem]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelGetListOfConfigurationsV2DataItem,
        List[ApimodelGetListOfConfigurationsV2DataItem],
        Dict[Any, ApimodelGetListOfConfigurationsV2DataItem],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "configId": "config_id",
            "configName": "config_name",
            "configType": "config_type",
            "deploymentStatus": "deployment_status",
            "editable": "editable",
            "isHidden": "is_hidden",
            "value": "value",
            "applyMask": "apply_mask",
            "description": "description",
            "source": "source",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "configId": True,
            "configName": True,
            "configType": True,
            "deploymentStatus": True,
            "editable": True,
            "isHidden": True,
            "value": True,
            "applyMask": False,
            "description": False,
            "source": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Iam Service."""

__version__ = "7.40.1"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .account_create_test_user_request_v4 import AccountCreateTestUserRequestV4
from .account_create_test_user_request_v4 import (
    AuthTypeEnum as AccountCreateTestUserRequestV4AuthTypeEnum,
)
from .account_create_test_user_response_v4 import AccountCreateTestUserResponseV4
from .account_create_test_users_request_v4 import AccountCreateTestUsersRequestV4
from .account_create_test_users_response_v4 import AccountCreateTestUsersResponseV4
from .account_create_user_request_v4 import AccountCreateUserRequestV4
from .account_create_user_request_v4 import (
    AuthTypeEnum as AccountCreateUserRequestV4AuthTypeEnum,
)
from .account_create_user_response_v4 import AccountCreateUserResponseV4
from .account_upgrade_headless_account_request_v4 import (
    AccountUpgradeHeadlessAccountRequestV4,
)
from .account_upgrade_headless_account_with_verification_code_forward_request_v4 import (
    AccountUpgradeHeadlessAccountWithVerificationCodeForwardRequestV4,
)
from .account_upgrade_headless_account_with_verification_code_request_v4 import (
    AccountUpgradeHeadlessAccountWithVerificationCodeRequestV4,
)
from .account_user_active_ban_response_v4 import AccountUserActiveBanResponseV4
from .account_user_info import AccountUserInfo
from .account_user_permissions_response_v4 import AccountUserPermissionsResponseV4
from .account_user_response_v4 import AccountUserResponseV4
from .accountcommon_allowed_permission import AccountcommonAllowedPermission
from .accountcommon_avatar_config import AccountcommonAvatarConfig
from .accountcommon_ban import AccountcommonBan
from .accountcommon_ban_reason import AccountcommonBanReason
from .accountcommon_ban_reason_v3 import AccountcommonBanReasonV3
from .accountcommon_ban_reasons import AccountcommonBanReasons
from .accountcommon_ban_reasons_v3 import AccountcommonBanReasonsV3
from .accountcommon_ban_v3 import AccountcommonBanV3
from .accountcommon_banned_by_v3 import AccountcommonBannedByV3
from .accountcommon_bans import AccountcommonBans
from .accountcommon_bans_v3 import AccountcommonBansV3
from .accountcommon_client_module_permission import AccountcommonClientModulePermission
from .accountcommon_client_permission import AccountcommonClientPermission
from .accountcommon_client_permission_set import AccountcommonClientPermissionSet
from .accountcommon_client_permission_v3 import AccountcommonClientPermissionV3
from .accountcommon_client_permissions import AccountcommonClientPermissions
from .accountcommon_client_permissions_v3 import AccountcommonClientPermissionsV3
from .accountcommon_client_selected_group import AccountcommonClientSelectedGroup
from .accountcommon_conflicted_user_platform_accounts import (
    AccountcommonConflictedUserPlatformAccounts,
)
from .accountcommon_country import AccountcommonCountry
from .accountcommon_country_age_restriction import AccountcommonCountryAgeRestriction
from .accountcommon_description import AccountcommonDescription
from .accountcommon_distinct_linked_platform_v3 import (
    AccountcommonDistinctLinkedPlatformV3,
)
from .accountcommon_distinct_platform_response_v3 import (
    AccountcommonDistinctPlatformResponseV3,
)
from .accountcommon_group_and_role_mapping_for_patch import (
    AccountcommonGroupAndRoleMappingForPatch,
)
from .accountcommon_group_and_role_mapping_for_update import (
    AccountcommonGroupAndRoleMappingForUpdate,
)
from .accountcommon_input_validation_description import (
    AccountcommonInputValidationDescription,
)
from .accountcommon_jwt_ban_v3 import AccountcommonJWTBanV3
from .accountcommon_list_users_with_platform_accounts_response import (
    AccountcommonListUsersWithPlatformAccountsResponse,
)
from .accountcommon_namespace_role import AccountcommonNamespaceRole
from .accountcommon_netflix_certificates import AccountcommonNetflixCertificates
from .accountcommon_override_role_permission import AccountcommonOverrideRolePermission
from .accountcommon_pagination import AccountcommonPagination
from .accountcommon_pagination_v3 import AccountcommonPaginationV3
from .accountcommon_permission import AccountcommonPermission
from .accountcommon_permission_group import AccountcommonPermissionGroup
from .accountcommon_permission_v3 import AccountcommonPermissionV3
from .accountcommon_permissions import AccountcommonPermissions
from .accountcommon_permissions_v3 import AccountcommonPermissionsV3
from .accountcommon_platform_account import AccountcommonPlatformAccount
from .accountcommon_platform_linking_history import AccountcommonPlatformLinkingHistory
from .accountcommon_platform_user_information_v3 import (
    AccountcommonPlatformUserInformationV3,
)
from .accountcommon_profile_update_config import AccountcommonProfileUpdateConfig
from .accountcommon_query_cursor import AccountcommonQueryCursor
from .accountcommon_registered_domain import AccountcommonRegisteredDomain
from .accountcommon_replace_role_permission import AccountcommonReplaceRolePermission
from .accountcommon_role import AccountcommonRole
from .accountcommon_role_manager import AccountcommonRoleManager
from .accountcommon_role_manager_v3 import AccountcommonRoleManagerV3
from .accountcommon_role_member import AccountcommonRoleMember
from .accountcommon_role_member_v3 import AccountcommonRoleMemberV3
from .accountcommon_role_v3 import AccountcommonRoleV3
from .accountcommon_simple_user_platform_info_v3 import (
    AccountcommonSimpleUserPlatformInfoV3,
)
from .accountcommon_sso_config import AccountcommonSsoConfig
from .accountcommon_sso_config_patch_req import AccountcommonSsoConfigPatchReq
from .accountcommon_tag_response import AccountcommonTagResponse
from .accountcommon_tags_get_response_v3 import AccountcommonTagsGetResponseV3
from .accountcommon_user_information_v3 import AccountcommonUserInformationV3
from .accountcommon_user_linked_platform import AccountcommonUserLinkedPlatform
from .accountcommon_user_linked_platform_v3 import AccountcommonUserLinkedPlatformV3
from .accountcommon_user_linked_platforms_response_v3 import (
    AccountcommonUserLinkedPlatformsResponseV3,
)
from .accountcommon_user_platform_info import AccountcommonUserPlatformInfo
from .accountcommon_user_platforms import AccountcommonUserPlatforms
from .accountcommon_user_search_by_platform_id_result import (
    AccountcommonUserSearchByPlatformIDResult,
)
from .accountcommon_user_search_result import AccountcommonUserSearchResult
from .accountcommon_user_with_linked_platform_accounts import (
    AccountcommonUserWithLinkedPlatformAccounts,
)
from .accountcommon_user_with_platform_accounts import (
    AccountcommonUserWithPlatformAccounts,
)
from .banned_by import BannedBy
from .bloom_filter_json import BloomFilterJSON
from .clientmodel_client_create_request import ClientmodelClientCreateRequest
from .clientmodel_client_creation_response import ClientmodelClientCreationResponse
from .clientmodel_client_creation_v3_request import ClientmodelClientCreationV3Request
from .clientmodel_client_response import ClientmodelClientResponse
from .clientmodel_client_template import ClientmodelClientTemplate
from .clientmodel_client_update_request import ClientmodelClientUpdateRequest
from .clientmodel_client_update_secret_request import (
    ClientmodelClientUpdateSecretRequest,
)
from .clientmodel_client_update_v3_request import ClientmodelClientUpdateV3Request
from .clientmodel_client_v3_response import ClientmodelClientV3Response
from .clientmodel_clients_update_request_v3 import ClientmodelClientsUpdateRequestV3
from .clientmodel_clients_v3_response import ClientmodelClientsV3Response
from .clientmodel_default_field_value import ClientmodelDefaultFieldValue
from .clientmodel_default_field_value_value import ClientmodelDefaultFieldValueValue
from .clientmodel_list_client_permission_set import ClientmodelListClientPermissionSet
from .clientmodel_list_templates_response import ClientmodelListTemplatesResponse
from .clientmodel_list_upsert_modules_request import ClientmodelListUpsertModulesRequest
from .clientmodel_module_package import ClientmodelModulePackage
from .clientmodel_permission_set_delete_group_request import (
    ClientmodelPermissionSetDeleteGroupRequest,
)
from .clientmodel_permission_set_upsert_request import (
    ClientmodelPermissionSetUpsertRequest,
)
from .clientmodel_upsert_permission_module_package_request import (
    ClientmodelUpsertPermissionModulePackageRequest,
)
from .clientmodel_v3_client_update_secret_request import (
    ClientmodelV3ClientUpdateSecretRequest,
)
from .legal_accepted_policies_request import LegalAcceptedPoliciesRequest
from .model_account_progression_info import ModelAccountProgressionInfo
from .model_add_user_role_v4_request import ModelAddUserRoleV4Request
from .model_admin_bulk_user_request import ModelAdminBulkUserRequest
from .model_age_restriction_request import ModelAgeRestrictionRequest
from .model_age_restriction_request_v3 import ModelAgeRestrictionRequestV3
from .model_age_restriction_response import ModelAgeRestrictionResponse
from .model_age_restriction_response_v3 import ModelAgeRestrictionResponseV3
from .model_assign_user_v4_request import ModelAssignUserV4Request
from .model_assigned_user_v4_response import ModelAssignedUserV4Response
from .model_authenticator_key_response_v4 import ModelAuthenticatorKeyResponseV4
from .model_backup_codes_response_v4 import ModelBackupCodesResponseV4
from .model_ban_create_request import ModelBanCreateRequest
from .model_ban_update_request import ModelBanUpdateRequest
from .model_bulk_account_type_update_request_v4 import (
    ModelBulkAccountTypeUpdateRequestV4,
)
from .model_bulk_ban_create_request_v3 import ModelBulkBanCreateRequestV3
from .model_bulk_unban_create_request_v3 import ModelBulkUnbanCreateRequestV3
from .model_check_availability_response import ModelCheckAvailabilityResponse
from .model_check_valid_user_id_request_v4 import ModelCheckValidUserIDRequestV4
from .model_config_value_response_v3 import ModelConfigValueResponseV3
from .model_country import ModelCountry
from .model_country_age_restriction_request import ModelCountryAgeRestrictionRequest
from .model_country_age_restriction_v3_request import (
    ModelCountryAgeRestrictionV3Request,
)
from .model_country_blacklist_request import ModelCountryBlacklistRequest
from .model_country_blacklist_response import ModelCountryBlacklistResponse
from .model_country_response import ModelCountryResponse
from .model_country_v3_response import ModelCountryV3Response
from .model_create_justice_user_response import ModelCreateJusticeUserResponse
from .model_cursor_get_user_request import ModelCursorGetUserRequest
from .model_cursor_get_user_response import ModelCursorGetUserResponse
from .model_cursor_get_user_response_data import ModelCursorGetUserResponseData
from .model_device_ban_request_v4 import ModelDeviceBanRequestV4
from .model_device_ban_response_v4 import ModelDeviceBanResponseV4
from .model_device_ban_update_request_v4 import ModelDeviceBanUpdateRequestV4
from .model_device_banned_response_v4 import ModelDeviceBannedResponseV4
from .model_device_bans_response_v4 import ModelDeviceBansResponseV4
from .model_device_id_decrypt_response_v4 import ModelDeviceIDDecryptResponseV4
from .model_device_response_v4 import ModelDeviceResponseV4
from .model_device_type_response_v4 import ModelDeviceTypeResponseV4
from .model_device_types_response_v4 import ModelDeviceTypesResponseV4
from .model_device_user_response_v4 import ModelDeviceUserResponseV4
from .model_device_users_response_v4 import ModelDeviceUsersResponseV4
from .model_devices_response_v4 import ModelDevicesResponseV4
from .model_disable_mfa_request import ModelDisableMFARequest
from .model_disable_user_request import ModelDisableUserRequest
from .model_email_update_request_v4 import ModelEmailUpdateRequestV4
from .model_enabled_factors_response_v4 import ModelEnabledFactorsResponseV4
from .model_failed_ban_unban_user_v3 import ModelFailedBanUnbanUserV3
from .model_field_update_allow_status import ModelFieldUpdateAllowStatus
from .model_forgot_password_request_v3 import ModelForgotPasswordRequestV3
from .model_forgot_password_response_v3 import ModelForgotPasswordResponseV3
from .model_forgot_password_without_namespace_request_v3 import (
    ModelForgotPasswordWithoutNamespaceRequestV3,
)
from .model_get_admin_users_response import ModelGetAdminUsersResponse
from .model_get_bulk_user_bans_request import ModelGetBulkUserBansRequest
from .model_get_link_headless_account_conflict_response import (
    ModelGetLinkHeadlessAccountConflictResponse,
)
from .model_get_profile_update_strategy_config_response import (
    ModelGetProfileUpdateStrategyConfigResponse,
)
from .model_get_publisher_user_response import ModelGetPublisherUserResponse
from .model_get_user_ban_summary_v3 import ModelGetUserBanSummaryV3
from .model_get_user_ban_v3_response import ModelGetUserBanV3Response
from .model_get_user_justice_platform_account_response import (
    ModelGetUserJusticePlatformAccountResponse,
)
from .model_get_user_mapping import ModelGetUserMapping
from .model_get_user_mapping_v3 import ModelGetUserMappingV3
from .model_get_users_response_with_pagination_v3 import (
    ModelGetUsersResponseWithPaginationV3,
)
from .model_input_validation_config_version import ModelInputValidationConfigVersion
from .model_input_validation_data import ModelInputValidationData
from .model_input_validation_data_public import ModelInputValidationDataPublic
from .model_input_validation_update_payload import ModelInputValidationUpdatePayload
from .model_input_validations_public_response import ModelInputValidationsPublicResponse
from .model_input_validations_response import ModelInputValidationsResponse
from .model_internal_config_response_v3 import ModelInternalConfigResponseV3
from .model_invitation_history_response import ModelInvitationHistoryResponse
from .model_invite_user_request_v3 import ModelInviteUserRequestV3
from .model_invite_user_request_v4 import ModelInviteUserRequestV4
from .model_invite_user_response_v3 import ModelInviteUserResponseV3
from .model_link_headless_account_request import ModelLinkHeadlessAccountRequest
from .model_link_platform_account_request import ModelLinkPlatformAccountRequest
from .model_link_platform_account_with_progression_request import (
    ModelLinkPlatformAccountWithProgressionRequest,
)
from .model_link_request_response import ModelLinkRequestResponse
from .model_linking_history_response_with_pagination_v3 import (
    ModelLinkingHistoryResponseWithPaginationV3,
)
from .model_list_assigned_users_v4_response import ModelListAssignedUsersV4Response
from .model_list_bulk_user_ban_response_v3 import ModelListBulkUserBanResponseV3
from .model_list_bulk_user_platforms_response import ModelListBulkUserPlatformsResponse
from .model_list_bulk_user_response import ModelListBulkUserResponse
from .model_list_email_address_request import ModelListEmailAddressRequest
from .model_list_invitation_histories_v4_response import (
    ModelListInvitationHistoriesV4Response,
)
from .model_list_role_v4_response import ModelListRoleV4Response
from .model_list_user_information_result import ModelListUserInformationResult
from .model_list_user_response_v3 import ModelListUserResponseV3
from .model_list_user_roles_v4_response import ModelListUserRolesV4Response
from .model_list_valid_user_id_response_v4 import ModelListValidUserIDResponseV4
from .model_login_allowlist_request import ModelLoginAllowlistRequest
from .model_login_allowlist_response import ModelLoginAllowlistResponse
from .model_login_histories_response import ModelLoginHistoriesResponse
from .model_namespace_invitation_history_user_v4_response import (
    ModelNamespaceInvitationHistoryUserV4Response,
)
from .model_namespace_role_request import ModelNamespaceRoleRequest
from .model_one_time_code_link_redirection_response import (
    ModelOneTimeCodeLinkRedirectionResponse,
)
from .model_permission_delete_request import ModelPermissionDeleteRequest
from .model_platform_defaults_response import ModelPlatformDefaultsResponse
from .model_platform_domain_delete_request import ModelPlatformDomainDeleteRequest
from .model_platform_domain_patch_request import ModelPlatformDomainPatchRequest
from .model_platform_domain_response import ModelPlatformDomainResponse
from .model_platform_domain_update_request import ModelPlatformDomainUpdateRequest
from .model_platform_user_id_request import ModelPlatformUserIDRequest
from .model_platform_user_id_request_v4 import ModelPlatformUserIDRequestV4
from .model_platform_user_information import ModelPlatformUserInformation
from .model_progression_info import ModelProgressionInfo
from .model_public_invite_user_request_v4 import ModelPublicInviteUserRequestV4
from .model_public_open_id_user_info_response import ModelPublicOpenIDUserInfoResponse
from .model_public_third_party_platform_info import ModelPublicThirdPartyPlatformInfo
from .model_public_user_information_response_v3 import (
    ModelPublicUserInformationResponseV3,
)
from .model_public_user_information_v3 import ModelPublicUserInformationV3
from .model_public_user_response import ModelPublicUserResponse
from .model_public_user_response_v3 import ModelPublicUserResponseV3
from .model_public_user_update_request_v3 import ModelPublicUserUpdateRequestV3
from .model_public_users_response import ModelPublicUsersResponse
from .model_re_auth_request import ModelReAuthRequest
from .model_remove_user_role_v4_request import ModelRemoveUserRoleV4Request
from .model_reset_password_request import ModelResetPasswordRequest
from .model_reset_password_request_v3 import ModelResetPasswordRequestV3
from .model_revoke_user_v4_request import ModelRevokeUserV4Request
from .model_role_admin_status_response import ModelRoleAdminStatusResponse
from .model_role_admin_status_response_v3 import ModelRoleAdminStatusResponseV3
from .model_role_create_request import ModelRoleCreateRequest
from .model_role_create_v3_request import ModelRoleCreateV3Request
from .model_role_managers_request import ModelRoleManagersRequest
from .model_role_managers_request_v3 import ModelRoleManagersRequestV3
from .model_role_managers_response import ModelRoleManagersResponse
from .model_role_managers_responses_v3 import ModelRoleManagersResponsesV3
from .model_role_members_request import ModelRoleMembersRequest
from .model_role_members_request_v3 import ModelRoleMembersRequestV3
from .model_role_members_response import ModelRoleMembersResponse
from .model_role_members_response_v3 import ModelRoleMembersResponseV3
from .model_role_names_response_v3 import ModelRoleNamesResponseV3
from .model_role_override_response import ModelRoleOverrideResponse
from .model_role_override_source_response import ModelRoleOverrideSourceResponse
from .model_role_override_stats_update_request import (
    ModelRoleOverrideStatsUpdateRequest,
)
from .model_role_override_update_request import ModelRoleOverrideUpdateRequest
from .model_role_permission_response_v3 import ModelRolePermissionResponseV3
from .model_role_response import ModelRoleResponse
from .model_role_response_v3 import ModelRoleResponseV3
from .model_role_response_with_managers import ModelRoleResponseWithManagers
from .model_role_response_with_managers_and_pagination_v3 import (
    ModelRoleResponseWithManagersAndPaginationV3,
)
from .model_role_response_with_managers_v3 import ModelRoleResponseWithManagersV3
from .model_role_update_request import ModelRoleUpdateRequest
from .model_role_update_request_v3 import ModelRoleUpdateRequestV3
from .model_role_v4_request import ModelRoleV4Request
from .model_role_v4_response import ModelRoleV4Response
from .model_search_users_by_platform_id_response import (
    ModelSearchUsersByPlatformIDResponse,
)
from .model_search_users_response import ModelSearchUsersResponse
from .model_search_users_response_with_pagination_v3 import (
    ModelSearchUsersResponseWithPaginationV3,
)
from .model_send_register_verification_code_request import (
    ModelSendRegisterVerificationCodeRequest,
)
from .model_send_verification_code_request import ModelSendVerificationCodeRequest
from .model_send_verification_code_request_v3 import ModelSendVerificationCodeRequestV3
from .model_send_verification_link_request import ModelSendVerificationLinkRequest
from .model_simple_profile_update_strategy_configs import (
    ModelSimpleProfileUpdateStrategyConfigs,
)
from .model_simple_user_ban import ModelSimpleUserBan
from .model_sso_platform_credential_request import ModelSSOPlatformCredentialRequest
from .model_sso_platform_credential_response import ModelSSOPlatformCredentialResponse
from .model_tag_create_request_v3 import ModelTagCreateRequestV3
from .model_tag_detail import ModelTagDetail
from .model_tag_update_request_v3 import ModelTagUpdateRequestV3
from .model_third_party_login_platform_credential_request import (
    ModelThirdPartyLoginPlatformCredentialRequest,
)
from .model_third_party_login_platform_credential_response import (
    ModelThirdPartyLoginPlatformCredentialResponse,
)
from .model_token_third_party_link_status_response import (
    ModelTokenThirdPartyLinkStatusResponse,
)
from .model_unlink_user_platform_request import ModelUnlinkUserPlatformRequest
from .model_update_permission_schedule_request import (
    ModelUpdatePermissionScheduleRequest,
)
from .model_update_profile_update_strategy_config_request import (
    ModelUpdateProfileUpdateStrategyConfigRequest,
)
from .model_update_user_deletion_status_request import (
    ModelUpdateUserDeletionStatusRequest,
)
from .model_update_user_status_request import ModelUpdateUserStatusRequest
from .model_upgrade_headless_account_request import ModelUpgradeHeadlessAccountRequest
from .model_upgrade_headless_account_v3_request import (
    ModelUpgradeHeadlessAccountV3Request,
)
from .model_upgrade_headless_account_with_verification_code_request import (
    ModelUpgradeHeadlessAccountWithVerificationCodeRequest,
)
from .model_upgrade_headless_account_with_verification_code_request_v3 import (
    ModelUpgradeHeadlessAccountWithVerificationCodeRequestV3,
)
from .model_user_active_ban_response import ModelUserActiveBanResponse
from .model_user_active_ban_response_v3 import ModelUserActiveBanResponseV3
from .model_user_ban_response import ModelUserBanResponse
from .model_user_ban_response_v3 import ModelUserBanResponseV3
from .model_user_ban_with_status import ModelUserBanWithStatus
from .model_user_base_info import ModelUserBaseInfo
from .model_user_bulk_update_request_v3 import ModelUserBulkUpdateRequestV3
from .model_user_create_request import ModelUserCreateRequest
from .model_user_create_request_v3 import ModelUserCreateRequestV3
from .model_user_create_response import ModelUserCreateResponse
from .model_user_create_response_v3 import ModelUserCreateResponseV3
from .model_user_deletion_status_response import ModelUserDeletionStatusResponse
from .model_user_i_ds_request import ModelUserIDsRequest
from .model_user_identity_update_request_v3 import ModelUserIdentityUpdateRequestV3
from .model_user_info_response import ModelUserInfoResponse
from .model_user_information import ModelUserInformation
from .model_user_input_validation_request import ModelUserInputValidationRequest
from .model_user_input_validation_response import ModelUserInputValidationResponse
from .model_user_invitation_history import ModelUserInvitationHistory
from .model_user_invitation_v3 import ModelUserInvitationV3
from .model_user_login_history_response import ModelUserLoginHistoryResponse
from .model_user_mfa_status_response_v4 import ModelUserMFAStatusResponseV4
from .model_user_mfa_token_response_v4 import ModelUserMFATokenResponseV4
from .model_user_password_update_request import ModelUserPasswordUpdateRequest
from .model_user_password_update_v3_request import ModelUserPasswordUpdateV3Request
from .model_user_permissions_response_v3 import ModelUserPermissionsResponseV3
from .model_user_platform_info import ModelUserPlatformInfo
from .model_user_platform_infos import ModelUserPlatformInfos
from .model_user_platform_link_histories import ModelUserPlatformLinkHistories
from .model_user_platform_link_history import ModelUserPlatformLinkHistory
from .model_user_platform_metadata import ModelUserPlatformMetadata
from .model_user_profile_update_allow_status import ModelUserProfileUpdateAllowStatus
from .model_user_public_info_response_v4 import ModelUserPublicInfoResponseV4
from .model_user_response import ModelUserResponse
from .model_user_response_v3 import ModelUserResponseV3
from .model_user_roles_v4_response import ModelUserRolesV4Response
from .model_user_state_response_v3 import ModelUserStateResponseV3
from .model_user_unban_create_request_v3 import ModelUserUnbanCreateRequestV3
from .model_user_update_request import ModelUserUpdateRequest
from .model_user_update_request_v3 import ModelUserUpdateRequestV3
from .model_user_verification_request import ModelUserVerificationRequest
from .model_user_verification_request_v3 import ModelUserVerificationRequestV3
from .model_user_with_platform_info import ModelUserWithPlatformInfo
from .model_users_platform_infos_request_v3 import ModelUsersPlatformInfosRequestV3
from .model_users_platform_infos_response import ModelUsersPlatformInfosResponse
from .model_users_update_request_v3 import ModelUsersUpdateRequestV3
from .model_valid_user_id_response_v4 import ModelValidUserIDResponseV4
from .model_validation_detail import ModelValidationDetail
from .model_validation_detail_public import ModelValidationDetailPublic
from .model_verification_code_response import ModelVerificationCodeResponse
from .model_verify_registration_code import ModelVerifyRegistrationCode
from .model_web_linking_response import ModelWebLinkingResponse
from .model_web_reauth_process_response import ModelWebReauthProcessResponse
from .oauthapi_revocation_list import OauthapiRevocationList
from .oauthcommon_as_metadata_response import OauthcommonASMetadataResponse
from .oauthcommon_jwk_key import OauthcommonJWKKey
from .oauthcommon_jwk_set import OauthcommonJWKSet
from .oauthcommon_user_revocation_list_record import OauthcommonUserRevocationListRecord
from .oauthmodel_client_register_response import OauthmodelClientRegisterResponse
from .oauthmodel_client_registration_request import OauthmodelClientRegistrationRequest
from .oauthmodel_country_location_response import OauthmodelCountryLocationResponse
from .oauthmodel_error_response import OauthmodelErrorResponse
from .oauthmodel_login_queue_ticket_response import OauthmodelLoginQueueTicketResponse
from .oauthmodel_one_time_linking_code_response import (
    OauthmodelOneTimeLinkingCodeResponse,
)
from .oauthmodel_one_time_linking_code_validation_response import (
    OauthmodelOneTimeLinkingCodeValidationResponse,
)
from .oauthmodel_platform_token_refresh_response_v3 import (
    OauthmodelPlatformTokenRefreshResponseV3,
)
from .oauthmodel_target_token_code_response import OauthmodelTargetTokenCodeResponse
from .oauthmodel_ticket_endpoint_action import OauthmodelTicketEndpointAction
from .oauthmodel_token_introspect_response import OauthmodelTokenIntrospectResponse
from .oauthmodel_token_response import OauthmodelTokenResponse
from .oauthmodel_token_response_v3 import OauthmodelTokenResponseV3
from .oauthmodel_token_third_party_response import OauthmodelTokenThirdPartyResponse
from .oauthmodel_token_with_device_cookie_response_v3 import (
    OauthmodelTokenWithDeviceCookieResponseV3,
)
from .oauthmodel_user_ban import OauthmodelUserBan
from .platform_dlc_agg_response import PlatformDLCAggResponse
from .platform_wallet_agg_response import PlatformWalletAggResponse
from .rest_error_response import RestErrorResponse
from .rest_error_response_with_conflicted_user_platform_accounts import (
    RestErrorResponseWithConflictedUserPlatformAccounts,
)
from .rest_permission import RestPermission
from .validation import Validation
from .validation_description import ValidationDescription



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ErrorEntity
from ..models import UserStatCycleItemPagingSlicedResult
from ..models import ValidationErrorEntity

from ..operations.user_statistic_cycle import GetUserStatCycleItems
from ..operations.user_statistic_cycle import GetUserStatCycleItems1
from ..operations.user_statistic_cycle import PublicListMyStatCycleItems


@same_doc_as(GetUserStatCycleItems)
def get_user_stat_cycle_items(
    cycle_id: str,
    user_id: str,
    is_public: Optional[bool] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[str] = None,
    stat_codes: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List user's statCycleItems by statCycle (getUserStatCycleItems)

    List user's statCycleItems by statCycle.
    Other detail info:

      *  Returns : stat cycle items

    Properties:
        url: /social/v1/admin/namespaces/{namespace}/users/{userId}/statCycles/{cycleId}/statCycleitems

        method: GET

        tags: ["UserStatisticCycle"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        cycle_id: (cycleId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        is_public: (isPublic) OPTIONAL bool in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL str in query

        stat_codes: (statCodes) OPTIONAL str in query

    Responses:
        200: OK - UserStatCycleItemPagingSlicedResult (successful operation)

        401: Unauthorized - ErrorEntity (20001: Unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        404: Not Found - ErrorEntity (12245: Stat cycle [{id}] cannot be found in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetUserStatCycleItems.create(
        cycle_id=cycle_id,
        user_id=user_id,
        is_public=is_public,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        stat_codes=stat_codes,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetUserStatCycleItems)
async def get_user_stat_cycle_items_async(
    cycle_id: str,
    user_id: str,
    is_public: Optional[bool] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[str] = None,
    stat_codes: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List user's statCycleItems by statCycle (getUserStatCycleItems)

    List user's statCycleItems by statCycle.
    Other detail info:

      *  Returns : stat cycle items

    Properties:
        url: /social/v1/admin/namespaces/{namespace}/users/{userId}/statCycles/{cycleId}/statCycleitems

        method: GET

        tags: ["UserStatisticCycle"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        cycle_id: (cycleId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        is_public: (isPublic) OPTIONAL bool in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL str in query

        stat_codes: (statCodes) OPTIONAL str in query

    Responses:
        200: OK - UserStatCycleItemPagingSlicedResult (successful operation)

        401: Unauthorized - ErrorEntity (20001: Unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        404: Not Found - ErrorEntity (12245: Stat cycle [{id}] cannot be found in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetUserStatCycleItems.create(
        cycle_id=cycle_id,
        user_id=user_id,
        is_public=is_public,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        stat_codes=stat_codes,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetUserStatCycleItems1)
def get_user_stat_cycle_items_1(
    cycle_id: str,
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[str] = None,
    stat_codes: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List user's statCycleItems by statCycle (getUserStatCycleItems_1)

    List user's statCycleItems by statCycle.
    Other detail info:

      *  Returns : stat cycle items

    Properties:
        url: /social/v1/public/namespaces/{namespace}/users/{userId}/statCycles/{cycleId}/statCycleitems

        method: GET

        tags: ["UserStatisticCycle"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        cycle_id: (cycleId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL str in query

        stat_codes: (statCodes) OPTIONAL str in query

    Responses:
        200: OK - UserStatCycleItemPagingSlicedResult (successful operation)

        401: Unauthorized - ErrorEntity (20001: Unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        404: Not Found - ErrorEntity (12245: Stat cycle [{id}] cannot be found in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetUserStatCycleItems1.create(
        cycle_id=cycle_id,
        user_id=user_id,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        stat_codes=stat_codes,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetUserStatCycleItems1)
async def get_user_stat_cycle_items_1_async(
    cycle_id: str,
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[str] = None,
    stat_codes: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List user's statCycleItems by statCycle (getUserStatCycleItems_1)

    List user's statCycleItems by statCycle.
    Other detail info:

      *  Returns : stat cycle items

    Properties:
        url: /social/v1/public/namespaces/{namespace}/users/{userId}/statCycles/{cycleId}/statCycleitems

        method: GET

        tags: ["UserStatisticCycle"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        cycle_id: (cycleId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL str in query

        stat_codes: (statCodes) OPTIONAL str in query

    Responses:
        200: OK - UserStatCycleItemPagingSlicedResult (successful operation)

        401: Unauthorized - ErrorEntity (20001: Unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        404: Not Found - ErrorEntity (12245: Stat cycle [{id}] cannot be found in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetUserStatCycleItems1.create(
        cycle_id=cycle_id,
        user_id=user_id,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        stat_codes=stat_codes,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicListMyStatCycleItems)
def public_list_my_stat_cycle_items(
    cycle_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[str] = None,
    stat_codes: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List user's statCycleItems by statCycle (publicListMyStatCycleItems)

    List user's statCycleItems by statCycle.
    Other detail info:

      *  Returns : stat cycle items

    Properties:
        url: /social/v1/public/namespaces/{namespace}/users/me/statCycles/{cycleId}/statCycleitems

        method: GET

        tags: ["UserStatisticCycle"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        cycle_id: (cycleId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL str in query

        stat_codes: (statCodes) OPTIONAL List[str] in query

    Responses:
        200: OK - UserStatCycleItemPagingSlicedResult (successful operation)

        401: Unauthorized - ErrorEntity (20001: Unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        404: Not Found - ErrorEntity (12245: Stat cycle [{id}] cannot be found in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicListMyStatCycleItems.create(
        cycle_id=cycle_id,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        stat_codes=stat_codes,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicListMyStatCycleItems)
async def public_list_my_stat_cycle_items_async(
    cycle_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[str] = None,
    stat_codes: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List user's statCycleItems by statCycle (publicListMyStatCycleItems)

    List user's statCycleItems by statCycle.
    Other detail info:

      *  Returns : stat cycle items

    Properties:
        url: /social/v1/public/namespaces/{namespace}/users/me/statCycles/{cycleId}/statCycleitems

        method: GET

        tags: ["UserStatisticCycle"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        cycle_id: (cycleId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL str in query

        stat_codes: (statCodes) OPTIONAL List[str] in query

    Responses:
        200: OK - UserStatCycleItemPagingSlicedResult (successful operation)

        401: Unauthorized - ErrorEntity (20001: Unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        404: Not Found - ErrorEntity (12245: Stat cycle [{id}] cannot be found in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicListMyStatCycleItems.create(
        cycle_id=cycle_id,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        stat_codes=stat_codes,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the Custom Service Manager."""

__version__ = "1.34.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .create_key_value_cluster_v2 import CreateKeyValueClusterV2
from .create_key_value_credential_v2 import CreateKeyValueCredentialV2
from .delete_key_value_cluster_v2 import DeleteKeyValueClusterV2
from .get_integration_app_key_181476 import GetIntegrationAppKeyValueClusterV2
from .get_key_value_cluster_l_62a4fb import GetKeyValueClusterLimitConfigV2
from .get_key_value_cluster_v2 import GetKeyValueClusterV2
from .get_list_integrated_app_4a9d12 import GetListIntegratedAppKeyValueClusterV2
from .integrate_app_key_value_d23a27 import IntegrateAppKeyValueClusterV2
from .list_key_value_cluster_v2 import ListKeyValueClusterV2
from .remove_integration_app__84d881 import RemoveIntegrationAppKeyValueClusterV2
from .update_key_value_cluster_v2 import UpdateKeyValueClusterV2
